@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
 #include "gs-window.h"
@@ -52,6 +53,7 @@ struct GSWindowPrivate
 
         guint      lock_enabled : 1;
         guint      logout_enabled : 1;
+        guint      user_switch_enabled : 1;
         guint64    logout_timeout;
 
         GtkWidget *box;
@@ -68,6 +70,8 @@ struct GSWindowPrivate
         gint       watch_id;
         gint       dialog_response;
 
+        GList     *key_events;
+
         GTimer    *timer;
 };
 
@@ -304,9 +308,24 @@ spawn_on_window (GSWindow *window,
 
         g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
                                                g_get_home_dir ()));
-        g_ptr_array_add (env, g_strdup_printf ("PATH=%s", g_getenv ("PATH")));
-        g_ptr_array_add (env, g_strdup_printf ("SESSION_MANAGER=%s", g_getenv ("SESSION_MANAGER")));
-        g_ptr_array_add (env, g_strdup_printf ("XAUTHORITY=%s", g_getenv ("XAUTHORITY")));
+        if (g_getenv ("PATH"))
+                g_ptr_array_add (env, g_strdup_printf ("PATH=%s",
+                                                       g_getenv ("PATH")));
+        if (g_getenv ("SESSION_MANAGER"))
+                g_ptr_array_add (env, g_strdup_printf ("SESSION_MANAGER=%s",
+                                                       g_getenv ("SESSION_MANAGER")));
+        if (g_getenv ("XAUTHORITY"))
+                g_ptr_array_add (env, g_strdup_printf ("XAUTHORITY=%s",
+                                                       g_getenv ("XAUTHORITY")));
+        if (g_getenv ("XAUTHLOCALHOSTNAME"))
+                g_ptr_array_add (env, g_strdup_printf ("XAUTHLOCALHOSTNAME=%s",
+                                                       g_getenv ("XAUTHLOCALHOSTNAME")));
+        if (g_getenv ("LANG"))
+                g_ptr_array_add (env, g_strdup_printf ("LANG=%s",
+                                                       g_getenv ("LANG")));
+        if (g_getenv ("LANGUAGE"))
+                g_ptr_array_add (env, g_strdup_printf ("LANGUAGE=%s",
+                                                       g_getenv ("LANGUAGE")));
         g_ptr_array_add (env, NULL);
 
         result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
@@ -370,11 +389,44 @@ plug_removed (GtkWidget *widget,
         return TRUE;
 }
 
+static void
+forward_key_events (GSWindow *window)
+{
+        window->priv->key_events = g_list_reverse (window->priv->key_events);
+
+        while (window->priv->key_events) {
+                GdkEventKey *event = window->priv->key_events->data;
+
+                gtk_window_propagate_key_event (GTK_WINDOW (window), event);
+
+                gdk_event_free ((GdkEvent *)event);
+                window->priv->key_events = g_list_delete_link (window->priv->key_events,
+                                                               window->priv->key_events);
+        }
+}
+
+static void
+remove_key_events (GSWindow *window)
+{
+        window->priv->key_events = g_list_reverse (window->priv->key_events);
+
+        while (window->priv->key_events) {
+                GdkEventKey *event = window->priv->key_events->data;
+
+                gdk_event_free ((GdkEvent *)event);
+                window->priv->key_events = g_list_delete_link (window->priv->key_events,
+                                                               window->priv->key_events);
+        }
+}
+
 static void
 socket_show (GtkWidget *widget,
              GSWindow  *window)
 {
         gtk_widget_child_focus (window->priv->socket, GTK_DIR_TAB_FORWARD);
+
+        /* send queued events to the dialog */
+        forward_key_events (window);
 }
 
 static void
@@ -440,6 +492,9 @@ gs_window_dialog_finish (GSWindow *window)
 
         g_spawn_close_pid (window->priv->pid);
         window->priv->pid = 0;
+
+        /* remove events for the case were we failed to show socket */
+        remove_key_events (window);
 }
 
 static gboolean
@@ -528,35 +583,43 @@ is_logout_enabled (GSWindow *window)
         return FALSE;
 }
 
+static gboolean
+is_user_switch_enabled (GSWindow *window)
+{
+  	return window->priv->user_switch_enabled;
+}
+
 static gboolean
 popup_dialog_idle (GSWindow *window)
 {
         gboolean  result;
-        char     *command;
+        char     *tmp;
+        GString  *command;
 
-        command = g_build_filename (LIBEXECDIR, "gnome-screensaver-dialog", NULL);
+        tmp = g_build_filename (LIBEXECDIR, "gnome-screensaver-dialog", NULL);
+        command = g_string_new (tmp);
+        g_free (tmp);
 
         if (is_logout_enabled (window)) {
-                char *cmd;
-
-                cmd = g_strdup_printf ("%s --enable-logout", command);
-                g_free (command);
-                command = cmd;
+                command = g_string_append (command, " --enable-logout");
+        }
+        if (is_user_switch_enabled (window)) {
+                command = g_string_append (command, " --enable-switch");
         }
 
         gs_window_clear (window);
         set_invisible_cursor (GTK_WIDGET (window)->window, FALSE);
 
         result = spawn_on_window (window,
-                                  command,
+                                  command->str,
                                   &window->priv->pid,
                                   (GIOFunc)command_watch,
                                   window,
                                   &window->priv->watch_id);
         if (!result)
-                g_warning ("Could not start command: %s", command);
+                g_warning ("Could not start command: %s", command->str);
 
-        g_free (command);
+        g_string_free (command, TRUE);
 
         window->priv->popup_dialog_idle_id = 0;
 
@@ -624,6 +687,15 @@ gs_window_set_logout_enabled (GSWindow *window,
         window->priv->logout_enabled = logout_enabled;
 }
 
+void
+gs_window_set_user_switch_enabled (GSWindow *window,
+                                   gboolean  user_switch_enabled)
+{
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        window->priv->user_switch_enabled = user_switch_enabled;
+}
+
 void
 gs_window_set_logout_timeout (GSWindow *window,
                               glong     logout_timeout)
@@ -728,14 +800,45 @@ gs_window_request_unlock_idle (GSWindow *window)
         return FALSE;
 }
 
+static void
+queue_key_event (GSWindow    *window,
+                 GdkEventKey *event)
+{
+        /* Eat the first return or space */
+        if (window->priv->key_events == NULL
+            && (event->keyval == GDK_Return
+                || event->keyval == GDK_space)) {
+                return;
+        }
+
+        window->priv->key_events = g_list_prepend (window->priv->key_events,
+                                                   gdk_event_copy ((GdkEvent *)event));
+}
+
 static gboolean
 gs_window_real_key_press_event (GtkWidget   *widget,
                                 GdkEventKey *event)
 {
+        gboolean catch_events = FALSE;
+
         /*g_message ("KEY PRESS state: %u keyval %u", event->state, event->keyval);*/
 
-        if (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0) {
-                GS_WINDOW (widget)->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, widget);
+        /* if we don't already have a socket then request an unlock */
+        if (! GS_WINDOW (widget)->priv->socket) {
+                if (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0) {
+                        GS_WINDOW (widget)->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, widget);
+                }
+
+                catch_events = TRUE;
+        } else {
+                if (! GTK_WIDGET_VISIBLE (GS_WINDOW (widget)->priv->socket)) {
+                        catch_events = TRUE;
+                }
+        }
+
+        /* Catch all keypresses up until the lock dialog is shown */
+        if (catch_events) {
+                queue_key_event (GS_WINDOW (widget), event);
         }
 
         if (GTK_WIDGET_CLASS (parent_class)->key_press_event)
@@ -748,7 +851,10 @@ static gboolean
 gs_window_real_motion_notify_event (GtkWidget      *widget,
                                     GdkEventMotion *event)
 {
-        if (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0) {
+
+        /* if we don't already have a socket then request an unlock */
+        if (! GS_WINDOW (widget)->priv->socket
+            && (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0)) {
                 GS_WINDOW (widget)->priv->request_unlock_idle_id = g_idle_add ((GSourceFunc)gs_window_request_unlock_idle, widget);
         }
 
@@ -913,6 +1019,9 @@ gs_window_finalize (GObject *object)
         if (window->priv->timer)
                 g_timer_destroy (window->priv->timer);
 
+        /* just in case they weren't removed */
+        remove_key_events (window);
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
